import { Zone, ZoneEventTileTeleport, ZoneEventTileType } from '@prisma/client'
import ZoneRepository from '../repositories/zoneRepository'
import ZoneService from '../services/zoneService'
import logger from '../utilities/logger'
import LoadedZone from '../models/loadedZone'
import zoneRepository from '../repositories/zoneRepository'
import { beforeEach } from 'node:test'
import prisma from '../utilities/prisma'

class ZoneManager {
  private loadedZones: LoadedZone[] = []

  // Method to initialize zoneEditor manager
  public async boot() {
    if (!(await ZoneRepository.getById(1))) {
      const zoneService = new ZoneService()
      await zoneService.createDemoZone()
    }

    const zones = await ZoneRepository.getAll()

    for (const zone of zones) {
      await this.loadZone(zone)
    }

    logger.info('Zone manager loaded')
  }

  // For now only current zone.
  public async getNeededAssets(zone: Zone, getMore = true): Promise<ZoneAssets> {
    const tiles: string[] = this.getUnique((JSON.parse(JSON.stringify(zone.tiles)) as string[][]).reduce((acc, val) => [...acc, ...val]));
    const objects = await zoneRepository.getObjects(zone.id);
    const eventTeleportTiles = await zoneRepository.getEventTeleportTiles(zone.id);
    let toZoneAssets = null;
    if(getMore) {
      for (const eventTeleportTile of eventTeleportTiles) {
        console.log(eventTeleportTile);
        console.log(eventTeleportTile.teleport.toZoneId);
        const toZone = await zoneRepository.getById(eventTeleportTile.teleport.toZoneId);
        console.log(toZone);
        if(toZone) {
          if(!toZoneAssets) {
            toZoneAssets = await this.getNeededAssets(toZone, false);
          } else {
            const newToZoneAssets = await this.getNeededAssets(toZone, false);
            toZoneAssets = {
              tiles: this.getUnique([...toZoneAssets.tiles, ...(newToZoneAssets ? newToZoneAssets.tiles : [])]),
              objects: this.getUnique([...toZoneAssets.objects, ...(newToZoneAssets ? newToZoneAssets.objects : [])]),
            } as ZoneAssets
          }
        }
      }
    }
    let mappedObjects = this.getUnique(objects.map(x => x.objectId));

    return {
      tiles: this.getUnique([...tiles, ...(toZoneAssets ? toZoneAssets.tiles : [])]),
      objects: this.getUnique([...mappedObjects, ...(toZoneAssets ? toZoneAssets.objects : [])]),
    } as ZoneAssets;
  }

  private getUnique<T>(array: T[]) {
    return [...new Set<T>(array)]
  }

  // Method to handle individual zoneEditor loading
  public async loadZone(zone: Zone) {
    const loadedZone = new LoadedZone(zone)
    this.loadedZones.push(loadedZone)
    await this.getNeededAssets(zone);
    logger.info(`Zone ID ${zone.id} loaded`)
  }

  // Method to handle individual zoneEditor unloading
  public unloadZone(zoneId: number) {
    this.loadedZones = this.loadedZones.filter((loadedZone) => loadedZone.getZone().id !== zoneId)
    logger.info(`Zone ID ${zoneId} unloaded`)
  }

  // Getter for loaded zones
  public getLoadedZones(): LoadedZone[] {
    return this.loadedZones
  }

  // Getter for zone by id
  public getZoneById(zoneId: number): LoadedZone | undefined {
    return this.loadedZones.find((loadedZone) => loadedZone.getZone().id === zoneId)
  }
}

export interface ZoneAssets {
  tiles: string[]
  objects: string[]
}

export default new ZoneManager()
